        // A class that maintains the common attributes of a UDP user.
        // This class looks after the socket attribute and
        // the InetAddress of the other end of the `connection'.
        // Typical clients extend this class to provide
        // the functionality applicable to their application.
        // It can also be used by server slave threads.

import java.net.*;
import java.io.*;

class UDPUser {
        // Typically used by a client to keep track
        // of the server.
    public UDPUser(String hostname) throws UnknownHostException, SocketException {
        // Obtain an InetAddress for the host.
        correspondent = InetAddress.getByName(hostname);
        // Create a datagram socket for sending.
        socket = new DatagramSocket();
    }

        // Typically used by a server slave to keep track
        // of the client.
    public UDPUser(InetAddress correspondent) throws SocketException {
        this.correspondent = correspondent;
        // Create a datagram socket for sending.
        socket = new DatagramSocket();
    }

        // Send packet and make sure that a reply is received.
        // Retry if necessary, but with a longer timeout.
        // Give up after too long, with an IOException.
    protected void sendAndReceive(DatagramPacket packet,DatagramPacket reply)
                        throws IOException {
        boolean keepTrying = true;
        // How long to wait for a reply. This is variable.
        int timeToWait = getRetryTime();
        // How many tries before giving up.
        int retriesLeft = 10;
        DatagramSocket socket = getSocket();
        while(keepTrying){
            socket.setSoTimeout(timeToWait);
            socket.send(packet);
            try{
                // Receive the reply.
                receive(reply);
                // Got it.
                keepTrying = false;
            }
            catch(InterruptedIOException e){
                if(retriesLeft > 0){
                    // Wait for longer next time.
                    timeToWait += timeToWait*2+1;
                    retriesLeft--;
                    System.out.println("Timeout: try to resend.");
                }
                else{
                    throw new IOException(
                      "Giving up: no reply received after multiple attempts.");
                }
            }
        }
        setRetryTime(timeToWait);
    }

        // Send a packet, but don't wait.
    protected void send(DatagramPacket packet) throws IOException {
        DatagramSocket socket = getSocket();
        socket.send(packet);
    }

        // Receive a packet.
    protected void receive(DatagramPacket packet)
                        throws IOException, InterruptedIOException {
        DatagramSocket socket = getSocket();
        socket.receive(packet);
    }

    protected void close(){
        getSocket().close();
    }

    protected InetAddress getCorrespondent(){
        return correspondent;
    }

    protected DatagramSocket getSocket(){
        return socket;
    }

    protected int getRetryTime(){
        return retryTime;
    }

    protected void setRetryTime(int t){
        retryTime = t;
    }

    // How long to wait before retrying a failed send.
    private int retryTime = 500;
    // With whom we are corresponding.
    private final InetAddress correspondent;
    // The socket we are using.
    private final DatagramSocket socket;
}
